@@ -30,6 +30,9 @@ SHOT_BATCH_SIZE = 2**18
 # Threshold size for sampling shots in measurements frequencies with custom operator
 SHOT_METROPOLIS_THRESHOLD = 100000
 
+# Max iterations for normalizing bistochastic matrices
+MAX_ITERATIONS = 50
+
 
 def raise_error(exception, message=None, args=None):
     """Raise exception with logging error.
