@@ -4,6 +4,7 @@ from typing import Dict, List, Optional, Tuple
 from qibo.config import raise_error
 from qibo.gates.abstract import Gate, ParametrizedGate
 
+# TODO: Make these dictionaries gate properties
 QASM_GATES = {
     "h": "H",
     "x": "X",
@@ -17,6 +18,7 @@ QASM_GATES = {
     "u3": "U3",
     "cx": "CNOT",
     "swap": "SWAP",
+    "iswap": "iSWAP",
     "fswap": "FSWAP",
     "rxx": "RXX",
     "ryy": "RYY",
@@ -38,9 +40,12 @@ PARAMETRIZED_GATES = {
     "rx",
     "ry",
     "rz",
+    "gpi",
+    "gpi2",
     "rxx",
     "ryy",
     "rzz",
+    "ms",
     "u1",
     "u2",
     "u3",
@@ -50,6 +55,56 @@ PARAMETRIZED_GATES = {
     "cu1",
     "cu3",
 }
+DRAW_LABELS = {
+    "h": "H",
+    "x": "X",
+    "y": "Y",
+    "z": "Z",
+    "s": "S",
+    "sdg": "SDG",
+    "t": "T",
+    "tdg": "TDG",
+    "rx": "RX",
+    "ry": "RY",
+    "rz": "RZ",
+    "gpi": "GPI",
+    "gpi2": "GPI2",
+    "u1": "U1",
+    "u2": "U2",
+    "u3": "U3",
+    "cx": "X",
+    "swap": "x",
+    "iswap": "i",
+    "cz": "Z",
+    "crx": "RX",
+    "cry": "RY",
+    "crz": "RZ",
+    "cu1": "U1",
+    "cu3": "U3",
+    "ccx": "X",
+    "id": "I",
+    "measure": "M",
+    "fsim": "f",
+    "generalizedfsim": "gf",
+    "rxx": "RXX",
+    "ryy": "RYY",
+    "rzz": "RZZ",
+    "Unitary": "U",
+    "fswap": "fx",
+    "ms": "MS",
+    "PauliNoiseChannel": "PN",
+    "KrausChannel": "K",
+    "UnitaryChannel": "U",
+    "ThermalRelaxationChannel": "TR",
+    "DepolarizingChannel": "D",
+    "ResetChannel": "R",
+    "PartialTrace": "PT",
+    "EntanglementEntropy": "EE",
+    "Norm": "N",
+    "Overlap": "O",
+    "Energy": "E",
+    "Fused Gate": "[]",
+}
 
 
 class H(Gate):
@@ -391,6 +446,9 @@ class RX(_Rn_):
         self.name = "rx"
         self._controlled_gate = CRX
 
+    def generator_eigenvalue(self):
+        return 0.5
+
 
 class RY(_Rn_):
     """Rotation around the Y-axis of the Bloch sphere.
@@ -418,6 +476,9 @@ class RY(_Rn_):
         self.name = "ry"
         self._controlled_gate = CRY
 
+    def generator_eigenvalue(self):
+        return 0.5
+
 
 class RZ(_Rn_):
     """Rotation around the Z-axis of the Bloch sphere.
@@ -443,6 +504,77 @@ class RZ(_Rn_):
         self.name = "rz"
         self._controlled_gate = CRZ
 
+    def generator_eigenvalue(self):
+        return 0.5
+
+
+class GPI(ParametrizedGate):
+    """The GPI gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        0 & e^{- i \\phi} \\\\
+        e^{i \\phi} & 0 \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+        phi (float): phase.
+        trainable (bool): whether gate parameters can be updated using
+            :meth:`qibo.models.circuit.AbstractCircuit.set_parameters`
+            (default is ``True``).
+    """
+
+    def __init__(self, q, phi, trainable=True):
+        super().__init__(trainable)
+        self.name = "gpi"
+        self.target_qubits = (q,)
+
+        self.parameter_names = "phi"
+        self.parameters = phi
+        self.nparams = 1
+
+        self.init_args = [q]
+        self.init_kwargs = {"phi": phi, "trainable": trainable}
+
+
+class GPI2(ParametrizedGate):
+    """The GPI2 gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & -i e^{- i \\phi} \\\\
+        -i e^{i \\phi} & 1 \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+        phi (float): phase.
+        trainable (bool): whether gate parameters can be updated using
+            :meth:`qibo.models.circuit.AbstractCircuit.set_parameters`
+            (default is ``True``).
+    """
+
+    def __init__(self, q, phi, trainable=True):
+        super().__init__(trainable)
+        self.name = "gpi"
+        self.target_qubits = (q,)
+
+        self.parameter_names = "phi"
+        self.parameters = phi
+        self.nparams = 1
+
+        self.init_args = [q]
+        self.init_kwargs = {"phi": phi, "trainable": trainable}
+
+    def _dagger(self) -> "Gate":
+        """"""
+        return self.__class__(self.target_qubits[0], self.parameters[0] + math.pi)
+
 
 class _Un_(ParametrizedGate):
     """Abstract class for defining the U1, U2 and U3 gates.
@@ -929,6 +1061,31 @@ class SWAP(Gate):
         self.init_args = [q0, q1]
 
 
+class iSWAP(Gate):
+    """The iswap gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 & 0 & 0 \\\\
+        0 & 0 & i & 0 \\\\
+        0 & i & 0 & 0 \\\\
+        0 & 0 & 0 & 1 \\\\
+        \\end{pmatrix}
+
+    Args:
+        q0 (int): the first qubit to be swapped id number.
+        q1 (int): the second qubit to be swapped id number.
+    """
+
+    def __init__(self, q0, q1):
+        super().__init__()
+        self.name = "iswap"
+        self.target_qubits = (q0, q1)
+        self.init_args = [q0, q1]
+
+
 class FSWAP(Gate):
     """The fermionic swap gate.
 
@@ -1026,7 +1183,7 @@ class GeneralizedfSim(ParametrizedGate):
         self.name = "generalizedfsim"
         self.target_qubits = (q0, q1)
 
-        self.parameter_names = ["u", "phi"]
+        self.parameter_names = ["unitary", "phi"]
         self.parameters = unitary, phi
         self.nparams = 5
 
@@ -1083,9 +1240,7 @@ class _Rnn_(ParametrizedGate):
 
 
 class RXX(_Rnn_):
-    """Parametric 2-qubit X \\otimes X interaction, or rotation about XX.
-
-    This is a symmetric gate.
+    """Parametric 2-qubit XX interaction, or rotation about XX-axis.
 
     Corresponds to the following unitary matrix
 
@@ -1112,7 +1267,7 @@ class RXX(_Rnn_):
 
 
 class RYY(_Rnn_):
-    """Parametric 2-qubit Y \\otimes Y interaction, or rotation about YY.
+    """Parametric 2-qubit YY interaction, or rotation about YY-axis.
 
     Corresponds to the following unitary matrix
 
@@ -1138,7 +1293,7 @@ class RYY(_Rnn_):
 
 
 class RZZ(_Rnn_):
-    """Parametric 2-qubit Z \\otimes Z interaction, or rotation about ZZ.
+    """Parametric 2-qubit ZZ interaction, or rotation about ZZ-axis.
 
     Corresponds to the following unitary matrix
 
@@ -1164,6 +1319,50 @@ class RZZ(_Rnn_):
         self.name = "rzz"
 
 
+class MS(ParametrizedGate):
+    """The Mølmer–Sørensen (MS) gate is a two qubit gate native to trapped ions.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 & 0 & -i e^{-i( \\phi_0 +  \\phi_1)} \\\\
+        0 & 1 & -i e^{-i( \\phi_0 -  \\phi_1)} \\\\
+        0 & -i e^{i( \\phi_0 -  \\phi_1)} & 1 & 0 \\\\
+        -i e^{i( \\phi_0 +  \\phi_1)} & 0 & 0 & 1 \\\\
+        \\end{pmatrix}
+
+    Args:
+        q0 (int): the first qubit to be swapped id number.
+        q1 (int): the second qubit to be swapped id number.
+        phi0 (float): first qubit's phase.
+        phi1 (float): second qubit's phase
+        trainable (bool): whether gate parameters can be updated using
+            :meth:`qibo.models.circuit.Circuit.set_parameters`
+            (default is ``True``).
+    """
+
+    # TODO: Check how this works with QASM.
+
+    def __init__(self, q0, q1, phi0, phi1, trainable=True):
+        super().__init__(trainable)
+        self.name = "ms"
+        self.target_qubits = (q0, q1)
+
+        self.parameter_names = ["phi0", "phi1"]
+        self.parameters = phi0, phi1
+        self.nparams = 2
+
+        self.init_args = [q0, q1]
+        self.init_kwargs = {"phi0": phi0, "phi1": phi1, "trainable": trainable}
+
+    def _dagger(self) -> "Gate":
+        """"""
+        q0, q1 = self.target_qubits
+        phi0, phi1 = self.parameters
+        return self.__class__(q0, q1, phi0 + math.pi, phi1)
+
+
 class TOFFOLI(Gate):
     """The Toffoli gate.
 
