@@ -1,57 +1,8 @@
 from typing import Dict, Optional, Tuple
 
-import sympy
-
 from qibo.config import raise_error
 from qibo.gates.abstract import Gate
-
-
-class MeasurementResult:
-    def __init__(self, qubits):
-        self.qubits = qubits
-        self.samples = []
-
-    def append(self, shot):
-        self.samples.append(shot)
-
-
-class MeasurementSymbol(sympy.Symbol):
-    """``sympy.Symbol`` connected to measurement results.
-
-    Used by :class:`qibo.gates.measurements.M` with ``collapse=True`` to allow
-    controlling subsequent gates from the measurement results.
-    """
-
-    _counter = 0
-
-    def __new__(cls, *args, **kwargs):
-        name = "m{}".format(cls._counter)
-        cls._counter += 1
-        return super().__new__(cls=cls, name=name)
-
-    def __init__(self, index, result):
-        self.index = index
-        self.result = result
-
-    def __getstate__(self):
-        return {"index": self.index, "result": self.result, "name": self.name}
-
-    def __setstate__(self, data):
-        self.index = data.get("index")
-        self.result = data.get("result")
-        self.name = data.get("name")
-
-    def outcome(self):
-        return self.result.samples[-1][self.index]
-
-    def evaluate(self, expr):
-        """Substitutes the symbol's value in the given expression.
-
-        Args:
-            expr (sympy.Expr): Sympy expression that involves the current
-                measurement symbol.
-        """
-        return expr.subs(self, self.outcome())
+from qibo.states import MeasurementResult
 
 
 class M(Gate):
@@ -94,7 +45,10 @@ class M(Gate):
         self.target_qubits = tuple(q)
         self.register_name = register_name
         self.collapse = collapse
-        self.result = None
+        self.result = MeasurementResult(self)
+        # list of measurement pulses implementing the gate
+        # relevant for experiments only
+        self.pulses = None
 
         self.init_args = q
         self.init_kwargs = {
@@ -109,7 +63,6 @@ class M(Gate):
                     NotImplementedError,
                     "Bitflip measurement noise is not " "available when collapsing.",
                 )
-            self.result = MeasurementResult(self.target_qubits)
 
         if p1 is None:
             p1 = p0
@@ -117,15 +70,6 @@ class M(Gate):
             p0 = p1
         self.bitflip_map = (self._get_bitflip_map(p0), self._get_bitflip_map(p1))
 
-    def get_symbols(self):
-        symbols = []
-        for i, q in enumerate(self.target_qubits):
-            symbols.append(MeasurementSymbol(i, self.result))
-        if len(self.target_qubits) > 1:
-            return symbols
-        else:
-            return symbols[0]
-
     @staticmethod
     def _get_bitflip_tuple(qubits: Tuple[int], probs: "ProbsType") -> Tuple[float]:
         if isinstance(probs, float):
@@ -170,45 +114,6 @@ class M(Gate):
             or sum(self.bitflip_map[1].values()) > 0
         )
 
-    @staticmethod
-    def einsum_string(qubits, nqubits, measuring=False):
-        """Generates einsum string for partial trace of density matrices.
-
-        Args:
-            qubits (list): Set of qubit ids that are traced out.
-            nqubits (int): Total number of qubits in the state.
-            measuring (bool): If True non-traced-out indices are multiplied and
-                the output has shape (nqubits - len(qubits),).
-                If False the output has shape 2 * (nqubits - len(qubits),).
-
-        Returns:
-            String to use in einsum for performing partial density of a
-            density matrix.
-        """
-        # TODO: Move this somewhere else (if it is needed at all)
-        from qibo.config import EINSUM_CHARS
-
-        if (2 - int(measuring)) * nqubits > len(EINSUM_CHARS):  # pragma: no cover
-            # case not tested because it requires large instance
-            raise_error(NotImplementedError, "Not enough einsum characters.")
-
-        left_in, right_in, left_out, right_out = [], [], [], []
-        for i in range(nqubits):
-            left_in.append(EINSUM_CHARS[i])
-            if i in qubits:
-                right_in.append(EINSUM_CHARS[i])
-            else:
-                left_out.append(EINSUM_CHARS[i])
-                if measuring:
-                    right_in.append(EINSUM_CHARS[i])
-                else:
-                    right_in.append(EINSUM_CHARS[i + nqubits])
-                    right_out.append(EINSUM_CHARS[i + nqubits])
-
-        left_in, left_out = "".join(left_in), "".join(left_out)
-        right_in, right_out = "".join(right_in), "".join(right_out)
-        return f"{left_in}{right_in}->{left_out}{right_out}"
-
     def add(self, gate):
         """Adds target qubits to a measurement gate.
 
@@ -229,33 +134,32 @@ class M(Gate):
         """"""
         raise_error(NotImplementedError, "Measurement gates cannot be controlled.")
 
-    @property
-    def matrix(self):
+    def asmatrix(self, backend):
         """"""
         raise_error(
             NotImplementedError, "Measurement gates do not have matrix representation."
         )
 
     def apply(self, backend, state, nqubits):
+        self.result.backend = backend
+        if not self.collapse:
+            return state
+
         qubits = sorted(self.target_qubits)
         # measure and get result
         probs = backend.calculate_probabilities(state, qubits, nqubits)
-        shot = backend.sample_shots(probs, 1)
-        # update the gate's result with the measurement outcome
-        binshot = backend.samples_to_binary(shot, len(qubits))[0]
-        self.result.backend = backend
-        self.result.append(binshot)
+        shot = self.result.add_shot(probs)
         # collapse state
         return backend.collapse_state(state, qubits, shot, nqubits)
 
     def apply_density_matrix(self, backend, state, nqubits):
+        self.result.backend = backend
+        if not self.collapse:
+            return state
+
         qubits = sorted(self.target_qubits)
         # measure and get result
         probs = backend.calculate_probabilities_density_matrix(state, qubits, nqubits)
-        shot = backend.sample_shots(probs, 1)
-        binshot = backend.samples_to_binary(shot, len(qubits))[0]
-        # update the gate's result with the measurement outcome
-        self.result.backend = backend
-        self.result.append(binshot)
+        shot = self.result.add_shot(probs)
         # collapse state
         return backend.collapse_density_matrix(state, qubits, shot, nqubits)
