@@ -1,5 +1,6 @@
 from qibo.config import raise_error
 from qibo.gates.abstract import Gate, ParametrizedGate, SpecialGate
+from qibo.gates.measurements import M
 
 
 class CallbackGate(SpecialGate):
@@ -37,7 +38,7 @@ class FusedGate(SpecialGate):
 
     def __init__(self, *q):
         super().__init__()
-        self.name = "fused"
+        self.name = "Fused Gate"
         self.target_qubits = tuple(sorted(q))
         self.init_args = list(q)
         self.qubit_set = set(q)
@@ -52,7 +53,7 @@ class FusedGate(SpecialGate):
     def from_gate(cls, gate):
         fgate = cls(*gate.qubits)
         fgate.append(gate)
-        if isinstance(gate, SpecialGate):
+        if isinstance(gate, (M, SpecialGate)):
             # special gates do not participate in fusion
             fgate.marked = True
         return fgate
@@ -75,9 +76,6 @@ class FusedGate(SpecialGate):
         else:
             self.gates.append(gate)
 
-    def __iter__(self):
-        return iter(self.gates)
-
     def _dagger(self):
         dagger = self.__class__(*self.init_args)
         for gate in self.gates[::-1]:
