@@ -1,5 +1,4 @@
 from qibo.config import raise_error
-from qibo.models.circuit import Circuit
 from qibo.models.evolution import StateEvolution
 
 
@@ -453,6 +452,7 @@ class QAOA:
         initial_p,
         initial_state=None,
         method="Powell",
+        mode=None,
         jac=None,
         hess=None,
         hessp=None,
@@ -464,7 +464,11 @@ class QAOA:
         compile=False,
         processes=None,
     ):
-        """Optimizes the variational parameters of the QAOA.
+        """Optimizes the variational parameters of the QAOA. A few loss functions are
+        provided for QAOA optimizations such as expected value (default), CVar which is introduced in
+        `Quantum 4, 256 <https://quantum-journal.org/papers/q-2020-04-20-256/>`_, and
+        Gibbs loss function which is introduced in
+        `PRR 2, 023074 (2020) <https://journals.aps.org/prresearch/abstract/10.1103/PhysRevResearch.2.023074>`_.
 
         Args:
             initial_p (np.ndarray): initial guess for the parameters.
@@ -472,6 +476,8 @@ class QAOA:
             method (str): the desired minimization method.
                 See :meth:`qibo.optimizers.optimize` for available optimization
                 methods.
+            mode (str): the desired loss function. The default is None. Alternatives are
+             "cvar", and "gibbs".
             jac (dict): Method for computing the gradient vector for scipy optimizers.
             hess (dict): Method for computing the hessian matrix for scipy optimizers.
             hessp (callable): Hessian of objective function times an arbitrary
@@ -491,6 +497,19 @@ class QAOA:
             returns the ``OptimizeResult``, for ``'cma'`` the
             ``CMAEvolutionStrategy.result``, and for ``'sgd'``
             the options used during the optimization.
+
+        Example:
+            .. testcode::
+
+                from qibo import hamiltonians
+
+                h = hamiltonians.XXZ(3)
+                qaoa = models.QAOA(h)
+                initial_p = [0.314, 0.22, 0.05, 0.59]
+                best, params, _ = qaoa.minimize(initial_p)
+                best, params, _ = qaoa.minimize(initial_p, mode="gibbs")
+                best, params, _ = qaoa.minimize(initial_p, mode="cvar")
+
         """
         if len(initial_p) % 2 != 0:
             raise_error(
@@ -505,7 +524,16 @@ class QAOA:
                 state = hamiltonian.backend.cast(state, copy=True)
             qaoa.set_parameters(params)
             state = qaoa(state)
-            return hamiltonian.expectation(state)
+            if mode is None:
+                return hamiltonian.expectation(state)
+            elif mode == "cvar":
+                from qibo.models.utils import cvar
+
+                return cvar(hamiltonian, state)
+            elif mode == "gibbs":
+                from qibo.models.utils import gibbs
+
+                return gibbs(hamiltonian, state)
 
         if method == "sgd":
             loss = lambda p, c, h, s: _loss(self.hamiltonian.backend.cast(p), c, h, s)
