@@ -13,14 +13,11 @@ from qibo.tests.utils import random_density_matrix, random_state
 def test_measurement_collapse(backend, nqubits, targets):
     initial_state = random_state(nqubits)
     c = models.Circuit(nqubits)
-    m = c.add(gates.M(*targets, collapse=True))
+    r = c.add(gates.M(*targets, collapse=True))
     final_state = backend.execute_circuit(c, np.copy(initial_state), nshots=1)[0]
-    if len(targets) > 1:
-        results = m[0].result.samples[0]
-    else:
-        results = m.result.samples[0]
+    samples = r.samples()[0]
     slicer = nqubits * [slice(None)]
-    for t, r in zip(targets, results):
+    for t, r in zip(targets, samples):
         slicer[t] = int(r)
     slicer = tuple(slicer)
     initial_state = initial_state.reshape(nqubits * (2,))
@@ -37,15 +34,12 @@ def test_measurement_collapse(backend, nqubits, targets):
 def test_measurement_collapse_density_matrix(backend, nqubits, targets):
     initial_rho = random_density_matrix(nqubits)
     c = models.Circuit(nqubits, density_matrix=True)
-    m = c.add(gates.M(*targets, collapse=True))
+    r = c.add(gates.M(*targets, collapse=True))
     final_rho = backend.execute_circuit(c, np.copy(initial_rho), nshots=1)[0]
 
-    if len(targets) > 1:
-        results = m[0].result.samples[0]
-    else:
-        results = m.result.samples[0]
+    samples = r.samples()[0]
     target_rho = np.reshape(initial_rho, 2 * nqubits * (2,))
-    for q, r in zip(targets, results):
+    for q, r in zip(targets, samples):
         r = int(r)
         slicer = 2 * nqubits * [slice(None)]
         slicer[q], slicer[q + nqubits] = 1 - r, 1 - r
@@ -71,8 +65,8 @@ def test_measurement_result_parameters(backend, effect, density_matrix):
     c = models.Circuit(4, density_matrix=density_matrix)
     if effect:
         c.add(gates.X(0))
-    output = c.add(gates.M(0, collapse=True))
-    c.add(gates.RX(1, theta=np.pi * output / 4))
+    r = c.add(gates.M(0, collapse=True))
+    c.add(gates.RX(1, theta=np.pi * r.symbols[0] / 4))
 
     target_c = models.Circuit(4, density_matrix=density_matrix)
     if effect:
@@ -88,9 +82,9 @@ def test_measurement_result_parameters_random(backend):
     initial_state = random_state(4)
     backend.set_seed(123)
     c = models.Circuit(4)
-    output = c.add(gates.M(1, collapse=True))
-    c.add(gates.RY(0, theta=np.pi * output / 5))
-    c.add(gates.RX(2, theta=np.pi * output / 4))
+    r = c.add(gates.M(1, collapse=True))
+    c.add(gates.RY(0, theta=np.pi * r.symbols[0] / 5))
+    c.add(gates.RX(2, theta=np.pi * r.symbols[0] / 4))
     final_state = backend.execute_circuit(
         c, initial_state=np.copy(initial_state), nshots=1
     )[0]
@@ -101,7 +95,7 @@ def test_measurement_result_parameters_random(backend):
     target_state = backend.execute_circuit(
         c, initial_state=np.copy(initial_state), nshots=1
     )[0]
-    if int(m.outcome()):
+    if int(m.symbols[0].outcome()):
         c = models.Circuit(4)
         c.add(gates.RY(0, theta=np.pi / 5))
         c.add(gates.RX(2, theta=np.pi / 4))
@@ -114,8 +108,8 @@ def test_measurement_result_parameters_repeated_execution(backend, use_loop):
     initial_state = random_state(4)
     backend.set_seed(123)
     c = models.Circuit(4)
-    output = c.add(gates.M(1, collapse=True))
-    c.add(gates.RX(2, theta=np.pi * output / 4))
+    r = c.add(gates.M(1, collapse=True))
+    c.add(gates.RX(2, theta=np.pi * r.symbols[0] / 4))
     if use_loop:
         final_states = []
         for _ in range(20):
@@ -134,7 +128,7 @@ def test_measurement_result_parameters_repeated_execution(backend, use_loop):
         c = models.Circuit(4)
         m = c.add(gates.M(1, collapse=True))
         target_state = backend.execute_circuit(c, np.copy(initial_state), nshots=1)[0]
-        if int(m.outcome()):
+        if int(m.symbols[0].outcome()):
             target_state = backend.apply_gate(
                 gates.RX(2, theta=np.pi / 4), target_state, 4
             )
@@ -148,9 +142,9 @@ def test_measurement_result_parameters_repeated_execution_final_measurements(bac
     initial_state = random_state(4)
     backend.set_seed(123)
     c = models.Circuit(4)
-    output = c.add(gates.M(1, collapse=True))
-    c.add(gates.RY(0, theta=np.pi * output / 3))
-    c.add(gates.RY(2, theta=np.pi * output / 4))
+    r = c.add(gates.M(1, collapse=True))
+    c.add(gates.RY(0, theta=np.pi * r.symbols[0] / 3))
+    c.add(gates.RY(2, theta=np.pi * r.symbols[0] / 4))
     c.add(gates.M(0, 1, 2, 3))
     result = backend.execute_circuit(c, initial_state=np.copy(initial_state), nshots=30)
     final_samples = result.samples(binary=False)
@@ -162,7 +156,7 @@ def test_measurement_result_parameters_repeated_execution_final_measurements(bac
         m = c.add(gates.M(1, collapse=True))
         target_state = backend.execute_circuit(c, np.copy(initial_state), nshots=1)[0]
         c = models.Circuit(4)
-        if int(m.outcome()):
+        if int(m.symbols[0].outcome()):
             c.add(gates.RY(0, theta=np.pi / 3))
             c.add(gates.RY(2, theta=np.pi / 4))
         c.add(gates.M(0, 1, 2, 3))
@@ -175,9 +169,9 @@ def test_measurement_result_parameters_multiple_qubits(backend):
     initial_state = random_state(4)
     backend.set_seed(123)
     c = models.Circuit(4)
-    output = c.add(gates.M(0, 1, 2, collapse=True))
-    c.add(gates.RY(1, theta=np.pi * output[0] / 5))
-    c.add(gates.RX(3, theta=np.pi * output[2] / 3))
+    r = c.add(gates.M(0, 1, 2, collapse=True))
+    c.add(gates.RY(1, theta=np.pi * r.symbols[0] / 5))
+    c.add(gates.RX(3, theta=np.pi * r.symbols[2] / 3))
     final_state = backend.execute_circuit(c, np.copy(initial_state), nshots=1)[0]
 
     backend.set_seed(123)
@@ -186,9 +180,9 @@ def test_measurement_result_parameters_multiple_qubits(backend):
     target_state = backend.execute_circuit(c, np.copy(initial_state), nshots=1)[0]
     # not including in coverage because outcomes are probabilistic and may
     # not occur for the CI run
-    if int(m[0].outcome()):  # pragma: no cover
+    if int(m.symbols[0].outcome()):  # pragma: no cover
         target_state = backend.apply_gate(gates.RY(1, theta=np.pi / 5), target_state, 4)
-    if int(m[2].outcome()):  # pragma: no cover
+    if int(m.symbols[2].outcome()):  # pragma: no cover
         target_state = backend.apply_gate(gates.RX(3, theta=np.pi / 3), target_state, 4)
     backend.assert_allclose(final_state, target_state)
 
@@ -197,10 +191,10 @@ def test_measurement_result_parameters_multiple_qubits(backend):
 def test_measurement_collapse_distributed(backend, accelerators, nqubits, targets):
     initial_state = random_state(nqubits)
     c = models.Circuit(nqubits, accelerators)
-    output = c.add(gates.M(*targets, collapse=True))
+    m = c.add(gates.M(*targets, collapse=True))
     result = backend.execute_circuit(c, np.copy(initial_state), nshots=1)
     slicer = nqubits * [slice(None)]
-    outcomes = [r.outcome() for r in output]
+    outcomes = [r.outcome() for r in m.symbols]
     for t, r in zip(targets, outcomes):
         slicer[t] = int(r)
     slicer = tuple(slicer)
@@ -216,12 +210,12 @@ def test_collapse_after_measurement(backend):
     qubits = [0, 2, 3]
     c = models.Circuit(5)
     c.add(gates.H(i) for i in range(5))
-    output = c.add(gates.M(*qubits, collapse=True))
+    m = c.add(gates.M(*qubits, collapse=True))
     c.add(gates.H(i) for i in range(5))
     final_state = backend.execute_circuit(c, nshots=1)[0]
 
     ct = models.Circuit(5)
-    bitstring = [r.outcome() for r in output]
+    bitstring = [r.outcome() for r in m.symbols]
     for i, r in zip(qubits, bitstring):
         if r:
             ct.add(gates.X(i))
