@@ -132,14 +132,28 @@ def test_add_measurement():
     g1 = gates.M(0, 2, register_name="a")
     g2 = gates.M(3, register_name="b")
     c.add([g1, g2])
-    assert c.measurement_gate is g1
+    assert len(c.queue) == 2
     assert c.measurement_tuples == {"a": (0, 2), "b": (3,)}
-    assert g1.target_qubits == (0, 2, 3)
+    assert g1.target_qubits == (0, 2)
     assert g2.target_qubits == (3,)
     with pytest.raises(KeyError):
         c.add(gates.M(4, register_name="b"))
 
 
+def test_add_measurement_collapse():
+    c = Circuit(3)
+    c.add(gates.X(0))
+    c.add(gates.M(0, 1))
+    c.add(gates.X(1))
+    c.add(gates.M(1))
+    c.add(gates.X(2))
+    c.add(gates.M(2))
+    assert len(c.queue) == 6
+    # assert that the first measurement was switched to collapse automatically
+    assert c.queue[1].collapse
+    assert len(c.measurements) == 2
+
+
 # :meth:`qibo.core.circuit.Circuit.fuse` is tested in `test_core_fusion.py`
 
 
@@ -213,11 +227,9 @@ def test_circuit_addition(measurements):
         c2.add(gates.M(1, register_name="b"))
 
     c3 = c1 + c2
-    assert c3.depth == 3
-    assert list(c3.queue) == [g1, g2, g3]
+    assert c3.depth == 3 + int(measurements)
     if measurements:
         assert c3.measurement_tuples == {"a": (0,), "b": (1,)}
-        assert c3.measurement_gate.target_qubits == (0, 1)
 
 
 def test_circuit_addition_errors():
@@ -336,7 +348,6 @@ def test_circuit_copy_with_measurements():
     c1.add(gates.M(0, 1, register_name="a"))
     c1.add(gates.M(3, register_name="b"))
     c2 = c1.copy()
-    assert c2.measurement_gate is c1.measurement_gate
     assert c2.measurement_tuples == {"a": (0, 1), "b": (3,)}
 
 
@@ -354,7 +365,6 @@ def test_circuit_invert(measurements):
         assert g1.target_qubits == g2.target_qubits
         assert g1.control_qubits == g2.control_qubits
     if measurements:
-        assert invc.measurement_gate.target_qubits == (0, 2)
         assert invc.measurement_tuples == {"register0": (0, 2)}
 
 
@@ -375,7 +385,6 @@ def test_circuit_decompose(measurements):
         assert g1.target_qubits == g2.target_qubits
         assert g1.control_qubits == g2.control_qubits
     if measurements:
-        assert decompc.measurement_gate.target_qubits == (0, 2)
         assert decompc.measurement_tuples == {"register0": (0, 2)}
 
 
@@ -405,7 +414,6 @@ def test_circuit_with_noise(measurements, noise_map):
         assert g1.target_qubits == g2.target_qubits
         assert g1.control_qubits == g2.control_qubits
     if measurements:
-        assert noisyc.measurement_gate.target_qubits == (0, 1)
         assert noisyc.measurement_tuples == {"register0": (0, 1)}
 
 
